# -*- coding: utf-8 -*-


# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Middleware initialization"""


from paste.cascade import Cascade
from paste.urlparser import StaticURLParser
from weberror.errormiddleware import ErrorMiddleware

from meningit.lib import wsgihelpers

from . import conf ## Filled by load_environnement in make_app
from .environment import load_environment
from . import urls


@wsgihelpers.wsgify.middleware
def environment_setter(req, app):
    """WSGI middleware that sets request-dependant environment."""
    urls.application_url = req.application_url
    return req.get_response(app)


def make_app(global_conf, **app_conf):
    """Create a WSGI application and return it

    ``global_conf``
        The inherited configuration for this application. Normally from
        the [DEFAULT] section of the Paste ini file.

    ``app_conf``
        The application's local configuration. Normally specified in
        the [app:<name>] section of the Paste ini file (where <name>
        defaults to main).
    """
    # Configure the environment and fill conf dictionary.
    load_environment(global_conf, app_conf)

    # Dispatch request to controllers.
    app = urls.make_router()

    # Init request-dependant environment
    app = environment_setter(app)

    # CUSTOM MIDDLEWARE HERE (filtered by error handling middlewares)

    # Handle Python exceptions
    if not conf['debug']:
        app = ErrorMiddleware(app, global_conf, **conf['errorware'])

    if conf['static_files']:
        # Serve static files
        static_app = StaticURLParser(conf['static_files_dir'])
        app = Cascade([static_app, app])

    return app

