# -*- coding: utf-8 -*-


# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Environment configuration"""


from ConfigParser import SafeConfigParser
import logging
import os
import sys

import mako.lookup
from paste.deploy.converters import asbool

from . import conf      # Empty dictionary
from .. import templates


config_dir = os.path.dirname(os.path.abspath(__file__))
app_dir = os.path.split(config_dir)[0]


def load_environment(global_conf, app_conf):
    """Configure the application environment."""
    conf.update({
        'app_conf': app_conf,
        'app_dir': app_dir,
        'cache_dir': os.path.join(os.path.dirname(app_dir), 'cache'),
        'debug': False,
        'global_conf': global_conf,
        'log_level': 'WARNING',
        'package_name': 'meningit',
        'repos_conf_file': os.path.join(global_conf['here'], 'repos.ini'),
        'static_files': True, # Whether this application serves its own static files
        'static_files_dir': os.path.join(app_dir, 'public'),
        })
    conf.update(global_conf)
    conf.update(app_conf)
    conf['debug'] = asbool(conf['debug'])
    conf['log_level'] = getattr(logging, conf['log_level'].upper())
    conf['static_files'] = asbool(conf['static_files'])

    # Configure logging.
    logging.basicConfig(level = conf['log_level'], stream = sys.stderr)

    errorware = conf.setdefault('errorware', {})
    errorware['debug'] = conf['debug']
    if not errorware['debug']:
        errorware['error_email'] = conf['email_to']
        errorware['error_log'] = conf.get('error_log', None)
        errorware['error_message'] = conf.get('error_message', 'An internal server error occurred')
        errorware['error_subject_prefix'] = conf.get('error_subject_prefix', 'MeninGIT Error: ')
        errorware['from_address'] = conf['from_address']
        errorware['smtp_server'] = conf.get('smtp_server', 'localhost')

    # Create the Mako TemplateLookup, with the default auto-escaping.
    templates.lookup = mako.lookup.TemplateLookup(
        default_filters = ['escape'],
        directories = [os.path.join(app_dir, 'templates')],
        imports = ['from webhelpers.html import escape'],
        input_encoding = 'utf-8',
        module_directory = os.path.join(conf['cache_dir'], 'templates'),
        )

    # Configure git repos.
    repos_conf = SafeConfigParser(dict(here = os.path.dirname(conf['repos_conf_file'])))
    repos_conf.read(conf['repos_conf_file'])
    conf['repos'] = {}
    for section in repos_conf.sections():
        conf['repos'][section] = {}
        for key, value in repos_conf.items(section):
            conf['repos'][section][key] = value

