# -*- coding: utf-8 -*-


# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Helpers for URLs"""


import re
import urlparse
import urllib

import webob.exc
from ..lib import wsgihelpers

application_url = None # Set to req.application_url as soon as MeninGIT application is called.
routes = []


def make_router():
    """Dispatch request to controllers."""
    from ..controllers import controllers

    for route in (
            ('GET', '^/$', controllers.index),
            ('GET', '^/commit$', controllers.commit),
            ('GET', '^/commit_index$', controllers.commit_index),
            ('GET', '^/diff$', controllers.diff),
            ('GET', '^/diff_from_index$', controllers.diff_from_index),
            ('GET', '^/erreur$', controllers.erreur),
            ('GET', '^/index$', controllers.index),
            ('GET', '^/view_commit$', controllers.view_commit),
            ):
        methods, regex, app = route[:3]
        if isinstance(methods, basestring):
            methods = (methods,)
        vars = route[3] if len(route) >= 4 else {}
        routes.append((methods, re.compile(regex), app, vars))
    return router


@wsgihelpers.wsgify
def router(req):
    split_path_info = req.path_info.split('/')
    assert not split_path_info[0]
    for methods, regex, app, vars in routes:
        if req.method in methods:
            match = regex.match(req.path_info)
            if match is not None:
                req.urlvars = match.groupdict()
                req.urlvars.update(vars)
                req.script_name += req.path_info[:match.end()]
                req.path_info = req.path_info[match.end():]
                return req.get_response(app)
    return webob.exc.HTTPNotFound("Path not found: %s" % req.path_info)


def url(ctx, *args, **kw):
    base_url = '/'
    if not kw.pop('full', False):
        # When a full URL is not requested, remove scheme and network location from it.
        base_url = urlparse.urlsplit(base_url).path
    fragments = list(args)
    method = kw.pop('method', 'GET')
    action_name = fragments.pop(0) if fragments else None
    if action_name == 'home':
        if method == 'GET':
            return "%s/" % (base_url)
    else:
        if action_name is None or action_name == 'index':
            assert not fragments
            assert not kw
            if method == 'GET':
                return base_url
        elif action_name in ('commit', 'erreur', 'repos'):
            assert not fragments
            message = kw.pop('message', getattr(ctx, 'message', None))
            if method == 'GET':
                return "%s%s?message=%s" % (base_url, action_name, message)
        elif action_name in ('diff', 'diff_from_index'):
            assert not fragments
            repo_name = kw.pop('repo_name', None)
            site = kw.pop('site', None)
            if method == 'GET':
                return "%s%s?%s" % (base_url, action_name, urllib.urlencode({'repo-name': repo_name, 'site': site}))

    raise Exception("Unknown url(%s, *%s, **%s): fragments = %s" % (ctx, args, kw, fragments))

