# -*- coding: utf-8 -*-


# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Mako templates rendering"""


import re
import simplejson


lookup = None # TemplateLookup inited by function load_environment.


def render(ctx, template_path, **kw):
    return indent(lookup.get_template(template_path).render_unicode(
        ctx = ctx,
        js = simplejson.dumps,
        N_ = lambda s: s,
        req = ctx.req,
        **kw).strip())


def render_def(ctx, template_path, def_name, **kw):
    return indent(lookup.get_template(template_path).get_def(def_name).render_unicode(
        ctx = ctx,
        js = simplejson.dumps,
        N_ = lambda s: s,
        req = ctx.req,
        **kw).strip())

def indent(render):
    opening_markup_re = re.compile("^ *<(?!(/|!)).+[^/]{1}>")
    closing_markup_re = re.compile("^.*</.+(?<!/)>")

    current_indent_level = 0
    render_return = ''

    for line in render.splitlines():
        line = line.strip()
        if closing_markup_re.match(line) and not opening_markup_re.match(line) and current_indent_level > 0:
            current_indent_level -= 1
        render_return = "%s%s%s\n" % (render_return, ' ' * 4 * current_indent_level, line)
        if opening_markup_re.match(line) and not closing_markup_re.match(line):
            current_indent_level += 1
    return render_return
