## -*- coding: utf-8 -*-


## MeninGIT -- Web-based wrapper of git
## By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
##
## Copyright (C) 2011 Easter-eggs
## http://wiki.infos-pratiques.org/wiki/MeninGIT
##
## This file is part of meninGIT.
##
## MeninGIT is free software; you can redistribute it and/or modify
## it under the terms of the GNU Affero General Public License as
## published by the Free Software Foundation, either version 3 of the
## License, or (at your option) any later version.
##
## MeninGIT is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Affero General Public License for more details.
##
## You should have received a copy of the GNU Affero General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.


<%!
import datetime

import difflib
import os

from dulwich.repo import Blob, Repo
from meningit.lib.utils import to_unicode, to_printable_date
from meningit.config import conf

def modified_files(repo_blobs, files_path, repo_path, callback):
    content = False
    if repo_blobs:
        for blob_info in repo_blobs:
            if blob_info[0] in files_path:
                file_path = "%s/%s" % (repo_path, blob_info[0])
                if not os.path.islink(file_path):
                    file = open(file_path)
                    blob = Blob.from_string(file.read())
                else:
                    blob = Blob.from_string(os.readlink(file_path))
                if blob.id != blob_info[1].id:
                    if not content:
                        content = True
                        yield content
                    yield callback(blob_info[1].data.split('\n'), blob.data.split('\n'), blob_info[0])
    if not content:
        yield content


def modified_files_from_changes(changes, repo, callback):
    content = False
    for change in changes:
        if None not in change[0]:
            if not content:
                content = True
                yield content
            old_blob = repo[change[2][0]]
            new_blob = repo[change[2][1]]
            yield callback(old_blob.data.split('\n'), new_blob.data.split('\n'), change[0][1])
    if not content:
        yield content
%>\


<%inherit file="site.mako"/>


<%def name="changes_file()" filter="trim">
    % if len(modif_list) > 0:
<div class="add">
    <h3>Fichiers ajoutés</h3>
        % for file in modif_list:
            % if file[1] == "create":
    <ins>${file[0]}</ins>
            % endif
        % endfor
</div>
<div class="remove">
    <h3>Fichiers supprimés</h3>
        % for file in modif_list:
            % if file[1] == "delete":
    <del>${file[0]}</del>
            % endif
        % endfor
</div>
    % else:
<div class="add">
    <h3>Aucun fichiers n'a été ajoutés</h3>
</div>
<div class="remove">
    <h3>Aucun fichiers n'a été supprimés</h3>
</div>
    % endif
</%def>

<%def name="file_diff(from_file, to_file, file_name)" filter="trim">
<%
    code = ""
    for line in difflib.context_diff(from_file, to_file, fromfile=file_name, tofile=file_name):
        code += "%s\n" % (line)
    code = to_unicode(code)
%>
<h3><a href="#">${file_name}</a></h3>
<pre>
${code | n}
</pre>
</%def>

<%def name="header()" filter="trim">
<div class="header">
    <h1>Gestion du dépôt "${repo_name}"</h1>
</div>
</%def>

<%def name="menu()" filter="trim">
<div class="menu">
    <a href="/diff_from_index?site=${site}&amp;repo-name=${repo_name}" class="link">Revenir à l'état courant</a>
    <a href="/" class="link">Selectionner un autre dépôt</a>
</div>

<div class="menu">
    <h2>Message de commit</h2>
    <form action="/commit_index" id="commit-form" method="get">
        <div>
            <textarea cols="40" rows="5" name="commit-message" id="commit-message"></textarea>
            <input type="hidden" id="repo-name" name="repo-name" value="${repo_name}" />
            <input type="hidden" id="site-name" name="site-name" value="${site}" />
            <input type="submit" class="link" value="Commit" />
            <div id="error-message"></div>
        </div>
    </form>
</div>

<div class="menu">
% if repo_commits:
    % if len(repo_commits) >= 10:
    <h2>Les 10 derniers Commits</h2>
    % else:
    <h2>Les ${len(repo_commits)} derniers Commits</h2>
    % endif
    <ul id="selectable-commits" class="selectable-commits">
    % for commit in repo_commits[:10]:
        <li class="ui-widget-content" id="n${commit.id}">
            <p>
                <span class="bold">${to_printable_date(commit.commit_time)}</span>
                <br />
            </p>
            <p>${to_unicode(commit.message).replace('\n', '<br />') | n}</p>
        </li>
    % endfor
    </ul>
% else:
    No Commits
% endif
</div>
</%def>


<div id="diff-body">
    <h2 class="state-view">Changements de l'état courant </h2>

    <div class="add-remove-files">
        ${changes_file()}
    </div>

    <div class="update">
<%
    if changes:
        repo = Repo(conf['repos'][site][repo_name])
        generator = modified_files_from_changes(changes, repo, file_diff)
    else:
        generator = modified_files(repo_blobs, files_path, repo_path, file_diff)
%>
    % if generator.next():
        <h3>Fichiers modifié</h3>

        <div id="accordion" class="file-diff">
        % for modif in generator:
            ${modif}
        % endfor
        </div>
    % else:
        <h3>Aucun fichier n'a été modifié</h3>
    % endif
    </div>
</div>


