#!/bin/bash

#set -x 
DEBUG=1

# For mail body with locale date format.
export LANG=fr_FR.UTF-8

#MENINGIT_CF="/etc/python-meninigt"
MENINGIT_CF="./"
INI_FILE="$MENINGIT_CF/sync-git.ini"

GIT_ROOT=/var/cache/git
GIT_MAILTO=pbrugier@easter-eggs.com

RSYNC_MODULE="etcgit"
export RSYNC_PASSWORD=Faik9iehAemieth5



if [ ! -f "$INI_FILE" ]
then
  echo "Error: ini file '${INI_FILE}' doesn't exist" >&2
  exit 0
fi

LINE_NUM=0

while read line
do

    ((LINE_NUM++))

    # Skip blank lines and comments
    if [ -z "$line" -o "${line:0:1}" = ";" -o "${line:0:1}" = "#" ]
    then
        continue
    fi

    # Section marker?
    line_rev=$(echo "$line" | rev)
    if [ "${line:0:1}" = "[" ] && [ "${line_rev:0:1}" = "]" ]
    then
        # Set SITE var to name of section (strip [ and ] from section marker)
        SITE="${line#[}"
        SITE="${SITE%]}"


        continue
    fi

    # Is there a server definition in the section

    VAR_VAL=$(echo "$line" | awk 'BEGIN { FS="=" } /^[a-zA-Z0-9._-]+[[:space:]]*=/ { print "Server Name: "$1,"Rsync name : ",$2; }')


    if [ -z "$VAR_VAL" ]
    then
        echo "Error: Invalid line:" >&2
        echo " ${LINE_NUM}: $line" >&2
        exit 1
    fi

    [ "$DEBUG" -gt 0 ] && echo -e "\n$VAR_VAL"

    
    # Get repository server name and real server name ands prot for rsync command
    SRV_NAME=$(echo "$line" |  cut -d "=" -f1 | sed -e 's/[[:space:]]*//g')
    SRV_VAL=$(echo "$line" |  cut -d "=" -f2 | sed -e 's/[[:space:]]*//g')

    # Convert $SITE, $SRV_NAME and $SRV_VAL to lower case
    SITE_LOWER=$(echo "$SITE" | tr '[:upper:]' '[:lower:]')
    SRV_NAME_LOWER=$(echo "$SRV_NAME" | tr '[:upper:]' '[:lower:]')
    SRV_VAL_LOWER=$(echo "$SRV_VAL" | tr '[:upper:]' '[:lower:]')

    [ "$DEBUG" -gt 0 ] && echo -e "SITE $SITE_LOWER - SRV $SRV_NAME_LOWER - RSYNC VAL $SRV_VAL_LOWER"

    # Rsync to Git repo now
    unset NEW

    HOST_DIR=$GIT_ROOT/$SITE_LOWER/$SRV_NAME_LOWER
    mkdir -p $HOST_DIR
    
    cd $HOST_DIR
    if [ ! -d "$HOST_DIR/.git" ]
    then
            NEW=ohyes
            git init
            echo "Configuration de $h" > .git/description
            cat $MENINGIT_CF/gitignore > .gitignore
    fi
    rsync --exclude '.git' --exclude '*~' --exclude .gitignore -azq --delete git@$SRV_VAL_LOWER::$RSYNC_MODULE/ .
    # Fake that we remove everything
    # This is needed, otherwise if we add a file, and then remove it, it's both
    # to-add and to-delete in the index, which is dumb.
    [ -z "$NEW" ] && git rm --cached --quiet -r -f .
    # Re-add everything
    git add .
    if [ -n "$NEW" ]
    then
        git commit --quiet -a -m "Initial import"
    else
        CHANGES=`git diff HEAD | wc -l`
        if [ $CHANGES -gt 0 ]
        then
            TMPFILE=`mktemp`
            if [ "`git config meningit.noautocommit`" = "yes" ]
            then
                echo "Please validate changes:" >> $TMPFILE
                echo "http://meningit.team-avionics.com/diff?repository_name=$SITE/$SRV_NAME_LOWER" >> $TMPFILE
                echo "-- " >> $TMPFILE
                git diff --stat -M HEAD >> $TMPFILE
                echo >> $TMPFILE
                git diff HEAD >> $TMPFILE
                mail -s "Summary of configuration changes on $SITE/$SRV_NAME_LOWER" "$GIT_MAILTO" < $TMPFILE
            else
                echo "Auto-commited changes follow." >> $TMPFILE
                echo "-- " >> $TMPFILE
                git diff --stat -M HEAD >> $TMPFILE
                echo >> $TMPFILE
                git diff HEAD >> $TMPFILE
                mail -s "Summary of AUTO commited configuration changes on $SITE/$SRV_NAME_LOWER" "$GIT_MAILTO" < $TMPFILE
                git commit --quiet -a -m "Auto-commit of modification @ `date`"
            fi
            rm -f $TMPFILE
        fi
    fi

    

done < $INI_FILE
